#include-once

; #INDEX# =======================================================================================================================
; Title .........: DllStructProtected
; AutoIt Version : 3.3.14.1
; Date ..........: 22.10.2015
; Modified ......: 20.10.2018 - Added buffering of kernel32.dll
; Language ......: Deutsch
; Description ...: Methoden um DllStructs vor dem AutoIt Aufrumdienst bei Referenzverlust zu schtzen.
; Author(s) .....: Mars
; ===============================================================================================================================

; #BEISPIEL# ====================================================================================================================
; Author ........: Mars
; Description ...: Zeigt die Unterschiede zu DllStructCreate
; ===============================================================================================================================

Global $__hDSP_Kernel32 = DllOpen('kernel32.dll')
OnAutoItExitRegister(__DSP_Exit)

#cs

BSP()

Func BSP()
	Local $sStr = 'int a; int b'

	; Ohne Schutz
	Local $vStr = DllStructCreate($sStr) ; Handelsbliche Struct
	Local $iPtr = DllStructGetPtr($vStr) ; Pointer ergaunern
	$vStr.a = 12345 ; Daten auffllen
	$vStr.b = 67890 ; Daten auffllen
	$vStr = 0 ; AutoIt ist ein Fuchs und Lscht die Struct
	$vStr = DllStructCreate($sStr, $iPtr) ; Wir erstellen an der Stelle der alten Struct eine neue
	ConsoleWrite($vStr.a & @CRLF & $vStr.b & @CRLF) ; Die Ausgabe der Daten kann jetzt von 12345 und 67890 abweichen

	; Mit Schutz
	Local $vStr = DllStructCreateProtected($sStr) ; Gesicherte Struct
	Local $iPtr = DllStructGetPtr($vStr) ; so
	$vStr.a = 12345 ; so
	$vStr.b = 67890 ; so
	$vStr = 0 ; AutoIt lscht die Struct auch dieses Mal, allerdings wird der Speicher nicht berschreiben
	$vStr = DllStructCreate($sStr, $iPtr) ; #OBACHT# Hier bitte nicht wieder mit DllStructCreateProtected arbeiten
	ConsoleWrite($vStr.a & @CRLF & $vStr.b & @CRLF) ; Die Ausgabe ist jetzt immer korrekt
	DllStructDelete($vStr) ; Struct lschen.

	; Wissenswertes:
	; - Es wird nicht die Struct, sondern die Speicherstelle geschtzt. Man sollte also nur EIN MAL via DllstructCreateProtected
	; dafr sorgen, dass die Speicherstelle sicher ist. Alle weiteren Structs die auf diese Stelle zugreifen sollten mit dem han
	; delsblichen DllStructCreate erzeugt werden.
	; - Die DllStructDelete Funktion nimmt Structs oder Pointer. Man muss also nicht erst eine Struct erstellen um den gesichert
	; en Speicher freizugeben.

EndFunc
#ce

; #FUNCTION# ====================================================================================================================
; Author ........: Mars
; Description ...: Da bei DllStructCreateProtected Structs nicht mehr automatisch gelscht werden, muss man das selbst tun.
; ===============================================================================================================================
Func DllStructDelete($pStr) ; Akzeptiert Structs sowie Pointer gleichermaen
	If IsDllStruct($pStr) Then $pStr = DllStructGetPtr($pStr)
	DllCall($__hDSP_Kernel32, 'ptr', 'GlobalFree', 'handle', $pStr)
EndFunc

; #FUNCTION# ====================================================================================================================
; Author ........: Mars
; Description ...: Funktioniert wie DllStructCreate mit dem Unterschied, dass bei Referenzverlust nichts gelscht wird.
; ===============================================================================================================================
Func DllStructCreateProtected($sStr)
	Local $vTmp = DllStructCreate($sStr) ; Nur um die Gre zu ermitteln
	Return DllStructCreate($sStr, DllCall($__hDSP_Kernel32, 'handle', 'GlobalAlloc', 'uint', 0x40, 'ulong_ptr', DllStructGetSize($vTmp))[0])
EndFunc

; #INTERNAL# ====================================================================================================================
; Author ........: Mars
; Description ...: Schliet die Kernel32.dll bei Skriptende
; ===============================================================================================================================
Func __DSP_Exit()
	DllClose($__hDSP_Kernel32)
EndFunc